//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     opt.h
// Purpose:    Options
// Last Change:12.09.2002
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2002 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	OPT_INCLUDED
#define OPT_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

//=============================================================================
// Enumeration Type for Options:
//=============================================================================

typedef enum {
	OPT_INPUT_FORMULAS,	// $i
	OPT_CLAUSES,		// $t - transformation into clause form
	OPT_HYPERRES,		// $h
	OPT_FACTS,		// $f
	OPT_RES_PROG,		// $r
	OPT_STATIC_COMPUTATION,	// $s
	OPT_DEFNEG_MODELS,	// $m
#ifdef VER_SEM_STABLE
	OPT_SEM_STABLE,		// $a
	OPT_COMPLETION,		// $c
#endif
	OPT_NIL			// invalid, must be last (array size)
} opt_e_t;

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// OPT_MAGIC: Magic number (identifies objects of this class for debugging).
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
static const long OPT_MAGIC =	0x4F505400L;	// 'OPT\0'
#endif

//=============================================================================
// Class for Program Options:
//=============================================================================

class opt_c {

//-----------------------------------------------------------------------------
// Static Member Functions (Class Methods):
//-----------------------------------------------------------------------------

	public:

	// Initialize this module:
	static void init(void);

	// Set an option (returns BOOL_FALSE if no valid option):
	static bool_t set_opt(char c);

	// Check whether option is set:
	static inline bool_t is_set(opt_e_t opt)
		{ CHECK(opt >= 0 && opt < OPT_NIL,
			"opt_c::is_set: illegal option");
		  CHECK(OptArray[opt] != 0,
			"opt_c::is_set: Module not initialized");
		  CHECK(OptArray[opt]->valid(),
			"opt_c::is_set: Option object is invalid");
		  return(OptArray[opt]->get_value());
		}

//-----------------------------------------------------------------------------
// Member Functions (Object Methods):
//-----------------------------------------------------------------------------

	public:

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity Check:
	public:
	bool_t	valid(void) const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "OPT_MAGIC"
#endif

//-----------------------------------------------------------------------------
// Auxillary Functions:
//-----------------------------------------------------------------------------

	private:
	
	// Constructor:
	opt_c(char letter, opt_e_t code);

	// Destructor (should never be called):
	~opt_c(void);

	// Get the value of an option (BOOL_TRUE if set):
	inline bool_t get_value(void) const
		{ CHECK(valid(), "opt_c::get_value: Bad object");
		  return(Value); }

	// Set the value of an option:
	inline void set_value(bool_t value)
		{ CHECK(valid(), "opt_c::set_value: Bad object");
		  CHECK(value==BOOL_TRUE || value==BOOL_FALSE,
			  "opt_c::set_value: Bad object");
		  Value = value; }

	// Get the atom for the option command:
	inline char get_letter(void) const
		{ CHECK(valid(), "opt_c::get_letter: Bad object");
		  return(Letter); }

//-----------------------------------------------------------------------------
// Static Data Members (Class Attributes):
//-----------------------------------------------------------------------------

	private:

	// Array of all option objects, index with enumeration type value:
	static opt_c* OptArray[OPT_NIL];

//-----------------------------------------------------------------------------
// Data Members (Object Attributes):
//-----------------------------------------------------------------------------

	private:

	// Atom for the command to set this option:
	char Letter;

	// Is this option set?
	bool_t Value;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//-----------------------------------------------------------------------------
// Declaration of Pointer Type:
//-----------------------------------------------------------------------------

typedef opt_c* opt_t;

//=============================================================================
// End of Include File:
//=============================================================================

#endif

